<?php
/*---------------------------------------------\
|											   |
| @Author:       Andrey Brykin (Drunya)        |
| @Version:      1.2                           |
| @Project:      AtomX CMS                     |
| @Package       AtomX CMS                     |
| @subpackege    VpsViewer class               |
| @copyright     ©Andrey Brykin                |
| @last mod.     2014/03/12                    |
|----------------------------------------------|
|											   |
| any partial or not partial extension         |
| CMS Fapos,without the consent of the         |
| author, is illegal                           |
|----------------------------------------------|
| Любое распространение                        |
| CMS Fapos или ее частей,                     |
| без согласия автора, является не законным    |
\---------------------------------------------*/



class Fps_Viewer_Manager
{

	protected $loader;
	
	protected $layout = 'default';

	protected $tokensParser;

	protected $treesParser;

	protected $compileParser;

	protected $nodesTree;

    private $markersData = array();
	
	

	public function __construct($loader = array())
	{
        $this->loader = new Fps_Viewer_Loader($loader);
		if (!empty($this->loader->layout)) $this->layout = $this->loader->layout;

		$this->tokensParser = new Fps_Viewer_TokensParser($this->loader);
		$this->treesParser = new Fps_Viewer_TreesParser($this->loader);
		$this->compileParser = new Fps_Viewer_CompileParser($this->loader);
	}


	
	public function setLayout($layout)
	{
		$this->layout = trim($layout);
	}
	


	public function view($fileName, $context = array())
	{
		$filePath = null;
        $cached = false;
		$fileSource = $this->getTemplateFile($fileName, $filePath);
        $filePath = str_replace(ROOT, '', $filePath);
	
        $fileSource = Plugins::intercept('before_view', $fileSource, $fileName);
		
		$start = getMicroTime();
		$data = $this->parseTemplate($fileSource, $context, $filePath, $cached);
        $took = getMicroTime() - $start;
		
		AtmDebug::addRow(
			array('Templates', 'Compile time', 'Cached'), 
			array($filePath, $took, ($cached ? 'From cache' : 'Compiled'))
		);
		
		return $data;
	}



	private function executeSource($source, $context)
	{
		$context = $this->prepareContext($context);
		ob_start();
		eval('?>' . $source);
		$output = ob_get_clean();
		return $output;
	}



	public function prepareContext($context)
	{
        $return = Plugins::intercept('markers_data', array_merge($this->markersData, $context));
		return $return;
	}



	private function getTemplateFile($fileName, &$returnPath = null)
	{
		$returnPath = $this->getTemplateFilePath($fileName);
		return file_get_contents($returnPath);
	}
	

	
    public function getTemplateFilePath($fileName)
    {
        $ext = strtolower(strrchr($fileName, '.'));
        if ($ext == '.msg') {
            $template = Config::read('mail_template', 'users');
            if (empty($template)) $template = 'default';
            $path = ROOT . '/data/mail/' . $template . '/' . $fileName;
        } else {
            $template = getTemplate();
            $path = ROOT . '/template/' . $template . '/html/' . '%s' . '/' . $fileName;
            if (file_exists(sprintf($path, $this->layout))) $path = sprintf($path, $this->layout);
            else $path = sprintf($path, 'default');
            $path = preg_replace('#([\\/])+#', '\\1', $path);
        }
        return $path;
    }
	
	
	
	
	public function parseTemplate($code, $context, $filePath = '', &$cached = false)
	{
        $key = md5($code);
        
        // preprocess snippets
		$this->loader->snippetsParser->setSource($code);
        $this->loader->snippetsParser->preprocess();

		if (
			$this->loader->cache && 
			Config::read('templates_cache') && 
			call_user_func($this->loader->cache['check'], $key)
		) {
			$sourceCode = call_user_func($this->loader->cache['read'], $key);
			$cached = true;
		} else {
			try {
                $this->treesParser->cleanStack();
                $tokens = $this->getTokens($code, $filePath);
                $nodes = $this->getTreeFromTokens($tokens);
                $this->compileParser->clean();
                $this->compileParser->setTmpClassName($this->getTmpClassName($code));
                $this->compile($nodes);
                $sourceCode = $this->compileParser->getOutput();
                //pr(h($sourceCode)); die();

            } catch (Exception $e) {
                throw new Exception('Parse template error on '
                    . (!empty($filePath) ? h($filePath) : 'Undefined') . ':'
                    . $e->getCode() . '. ' . $e->getMessage());
            }
			
			call_user_func($this->loader->cache['write'], $sourceCode, $key);
		}
        
		$output = $this->executeSource($sourceCode, $context);
		
        // replace snippets markers
        $output = $this->loader->snippetsParser->replace($output);

		return $output;
	}
	
	
	
	
	private function getTmpClassName($code)
	{
		return 'Fps_Viewer_Template_' . md5($code . rand());
	}
	


    public function setMarkers($markers)
    {
        $this->markersData = array_merge($this->markersData, $markers);
    }


	
	
	public function getTokens($code, $filePath = '')
	{
		return $this->tokensParser->parseTokens($code, $filePath);
	}
	
	
	
	
	public function getTreeFromTokens($tokens)
	{
		return $this->treesParser->parse($tokens);
	}
	
	
	
	public function compile($nodes)
	{
		return $this->compileParser->compile($nodes);
	}
	
	// для совместимости со старым вызовом
    public function setModuleTitle() {
    }
}